﻿using System.Collections.ObjectModel;
using System.Windows.Interactivity;
using Infragistics.DragDrop;
using Infragistics.Windows;
using Infragistics.Windows.DataPresenter;

namespace DragDropRecordsBetweenGrids
{
  /// <summary>
  /// A behavior to handle the <see cref="XamDataGrid.AssigningFieldLayoutToItem"/> event.
  /// </summary>
  public class GridRowShiftingBehavior : Behavior<DragSource>
  {
    #region Behaviour Overrides

    /// <summary>
    /// On behavior getting attached,
    /// </summary>
    protected override void OnAttached()
    {
      var attachedElement = AssociatedObject;
      attachedElement.Drop += AttachedElement_Drop;
    }

    /// <summary>
    /// While the behavior is getting detached.
    /// </summary>
    protected override void OnDetaching()
    {
      base.OnDetaching();
      var attachedElement = AssociatedObject;
      attachedElement.Drop -= AttachedElement_Drop;
    }
    #endregion

    #region Private Handlers

    /// <summary>
    /// Attached element being dropped.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    void AttachedElement_Drop(object sender, DropEventArgs e)
    {
      var recordPresenter = Utilities.GetAncestorFromType(e.DragSource, typeof(DataRecordPresenter), false) as DataRecordPresenter;
      var dataPresenter = Utilities.GetAncestorFromType(recordPresenter, typeof(XamDataGrid), false) as XamDataGrid;

      if (recordPresenter != null)
      {
        var dragRecord = recordPresenter.Record as DataRecord;
        var dataRecordPresenter = Utilities.GetAncestorFromType(e.DropTarget, typeof (DataRecordPresenter), false) as DataRecordPresenter;
        if (dataRecordPresenter != null)
        {
          var dropRecord = dataRecordPresenter.Record as DataRecord;

          if (dragRecord != null && dropRecord != null && 
            (dragRecord.DataPresenter == dropRecord.DataPresenter))
          {
            int insertIndex = dropRecord.DataItemIndex;

            var data = dragRecord.DataItem as ChartData;
            var collection = dragRecord.DataPresenter.DataSource as ObservableCollection<ChartData>;
            if (collection != null)
              collection.Remove(data);
            var datas = dropRecord.DataPresenter.DataSource as ObservableCollection<ChartData>;
            if (datas != null)
              datas.Insert(insertIndex, data);
          }
        }
      }
    }

    #endregion
  }
}
